def add(a, b):
    return a + b


def is_positive(n):
    if n > 0:
        return True
    return False


def greet(name):
    return "Hello, " + name


def untested_func():
    return "this function has no tests"
